import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  EventEmitter,
  Input,
  OnChanges,
  OnDestroy,
  OnInit,
  Output,
  SimpleChanges,
  TemplateRef,
  ViewChild,
} from "@angular/core";
import {
  CalendarItemNOA11Y,
  DayTemplateContextNOA11Y,
  DayViewModel,
  MonthViewModel,
  NgbDataTemplateStruct,
  NgbDate,
  NgbDatepickerI18nNOA11Y,
  NgbDatepickerServiceNOA11Y,
  WeekViewModel
} from "@isp/xdce-widget";
import { Subscription } from "rxjs";

@Component({
  selector: "ngb-datepicker-routing-view",
  host: { class: "d-block" },
  templateUrl: './datepicker-routing-view.html',
  styleUrls: ["./datepicker-routing-view.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class NgbFidDatepickerRoutingViewNOA11Y
  implements OnInit, OnChanges, OnDestroy
{
  @Input() dayTemplate: TemplateRef<DayTemplateContextNOA11Y>;
  @Input() monthTemplate: TemplateRef<DayTemplateContextNOA11Y>;
  @Input() yearsTemplate: TemplateRef<DayTemplateContextNOA11Y>;
  @Input() currentTemplate: number;
  @Input() mesi: any;
  @Input() years: any;
  @Input() disabledWeekend: boolean;

  @Input() date: any;
  @Input() dataTemplate: NgbDataTemplateStruct;

  @Input() month: MonthViewModel;

  @Input() outsideDays: "visible" | "hidden" | "collapsed";
  @Input() showWeekdays;
  @Input() showWeekNumbers;

  @Input() startDateEnabled: any;
  @Input() endDateEnabled: any;

  @Input() isToday: boolean;
  @Input() isWeekend: boolean;
  @Input() taxDeadlines: any;

  @Output() select = new EventEmitter<NgbDate>();
  @Output() changeTemplate = new EventEmitter<NgbDataTemplateStruct>();

  @ViewChild("ispDatepickerRoutingView") routingView: ElementRef;

  currentYear: number;
  currentMonth: number;
  currentDay: number;
  sub: Subscription;

  constructor(
    public i18n: NgbDatepickerI18nNOA11Y,
    private cdr: ChangeDetectorRef,
    private service: NgbDatepickerServiceNOA11Y
  ) {}
  ngOnInit() {
    // alert("in select in routing : " + this.startDateEnabled)
    // alert("in select in routing : " + this.endDateEnabled)
    // this.currentYear = this.date && this.date.year;
    // this.currentMonth = this.date && this.date.month;
    // console.info('routing init', this.currentYear, this.currentMonth);
    this.dataTemplate.day = this.date && this.date.day;
    this.dataTemplate.month = this.date && this.date.month;
    this.dataTemplate.year = this.date && this.date.year;
    const model = this.service._model$.getValue();
    if (model && model.selectedDate) {
      this.currentDay = model.selectedDate.day;
      this.currentMonth = model.selectedDate.month;
      this.currentYear = model.selectedDate.year;
    }
    // this.sub = this.service.model$.subscribe((model) => {
    //   if(model.selectedDate) {
    //     this.currentDay = model.selectedDate.day;
    //     this.currentMonth = model.selectedDate.month;
    //     this.currentYear = model.selectedDate.year;
    //   }
    // })
    // console.info('routing init', this.date, this.dataTemplate);
  }

  ngOnDestroy(): void {
    // if (this.sub) {
    //   this.sub.unsubscribe();
    // }
  }

  ngOnChanges(changes: SimpleChanges) {
    if (changes.years) {
      this.years = [...changes.years.currentValue];
    }
    if (changes.currentTemplate || changes.dataTemplate) {
      this.resetMesi();
      this.resetAnni();
      const model = this.service._model$.getValue();
      if (model && model.selectedDate) {
        this.currentDay = model.selectedDate.day;
        this.currentMonth = model.selectedDate.month;
        this.currentYear = model.selectedDate.year;
      }
    }
    this.cdr.detectChanges();
  }

  doSelect(day: DayViewModel, $event: any) {
    if (!this.abilitaGiorni(day.date)) {
      return;
    }

    if (this.dataTemplate.valid) {
      // il day si prende quello selezionato e non quello del dataTemplate
      // day.date.day = this.dataTemplate.day;
      day.date.month = this.dataTemplate.month;
      day.date.year = this.dataTemplate.year;
      this.currentYear = day.date.year;
      this.currentMonth = day.date.month;
      this.currentDay = day.date.day;
      this.select.emit(NgbDate.from(day.date));
    } else {
      if (!day.context.disabled && !this.isHidden(day)) {
        this.currentYear = day.date.year;
        this.currentMonth = day.date.month;
        this.currentDay = day.date.day;
        this.select.emit(NgbDate.from(day.date));
      }
    }
    $event.stopPropagation();
  }

  abilitaGiorni(data) {
    let buffer = new Date(data.year, data.month - 1, data.day);

    let startDate = new Date(
      this.startDateEnabled.year,
      this.startDateEnabled.month - 1,
      this.startDateEnabled.day
    );

    let endDate = new Date(
      this.endDateEnabled.year,
      this.endDateEnabled.month - 1,
      this.endDateEnabled.day
    );

    return startDate <= buffer && buffer <= endDate;
  }

  abilitaMesi(data) {
    return (
      ((this.startDateEnabled.year == this.dataTemplate.year &&
        this.startDateEnabled.month <= data.number) ||
        (data.number < this.startDateEnabled.month &&
          this.startDateEnabled.year < this.dataTemplate.year) ||
        (this.startDateEnabled.month <= data.number &&
          this.startDateEnabled.year <= this.dataTemplate.year)) &&
      ((data.number <= this.endDateEnabled.month &&
        this.dataTemplate.year <= this.endDateEnabled.year) ||
        (data.number > this.endDateEnabled.month &&
          this.dataTemplate.year < this.endDateEnabled.year))
    );
  }

  abilitaAnni(data) {
    return (
      this.startDateEnabled.year <= data.number &&
      data.number <= this.endDateEnabled.year
    );
  }

  doSelectMonth(mese: any, $event: any) {
    if (!this.abilitaMesi(mese)) {
      return;
    }

    this.currentTemplate = 0;
    this.dataTemplate.template = this.currentTemplate;

    // reset giorni
    this.month.weeks.forEach((week) => {
      week.days.forEach((day) => {
        day.context.selected =
          day.date.day === this.currentDay &&
          day.date.month === this.currentMonth &&
          day.date.year === this.currentYear;
      });
    });
    this.resetMesi();

    this.dataTemplate.month = mese.number;
    this.dataTemplate.valid = true;
    this.changeTemplate.emit(this.dataTemplate);
    $event.stopPropagation();
  }

  doSelectYear(year: any, $event: any) {
    if (!this.abilitaAnni(year)) {
      return;
    }

    this.currentTemplate = 1;
    this.dataTemplate.template = this.currentTemplate;

    // resetta mesi e anno
    this.resetMesi();

    this.dataTemplate.year = year.number;
    this.dataTemplate.valid = true;
    this.changeTemplate.emit(this.dataTemplate);
    $event.stopPropagation();
  }

  isCollapsed(week: WeekViewModel) {
    return (
      this.outsideDays === "collapsed" &&
      week.days[0].date.month !== this.month.number &&
      week.days[week.days.length - 1].date.month !== this.month.number
    );
  }

  isHiddenWeek(week: WeekViewModel) {
    let counter = 0;
    for (let i = 0; i < week.days.length; i++) {
      if (this.isHidden(week.days[i])) {
        counter++;
      }
    }
    return counter === 7;
  }

  isHidden(day: DayViewModel) {
    return (
      (this.outsideDays === "hidden" || this.outsideDays === "collapsed") &&
      this.month.number !== day.date.month
    );
  }

  getRoutingViewHeight() {
    return this.routingView.nativeElement.offsetHeight;
  }

  getRoutingViewWidth() {
    return this.routingView.nativeElement.offsetWidth;
  }

  private resetMesi() {
    let months: CalendarItemNOA11Y[][] = [];
    months = this.mesi.map((row) => {
      return row.map((m) => {
        const res = { ...m };
        res.selected =
          res.number === this.currentMonth &&
          this.dataTemplate.year === this.currentYear;
        return res;
      });
    });
    this.mesi = months;
  }

  private resetAnni() {
    let yrs: CalendarItemNOA11Y[][] = [];
    yrs = this.years.map((row) => {
      return row.map((y) => {
        const res = { ...y };
        res.selected = res.number === this.currentYear;
        return res;
      });
    });
    this.years = yrs;
  }
}
