import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  EventEmitter,
  HostListener,
  Input,
  OnChanges,
  OnDestroy,
  OnInit,
  Output,
  SimpleChanges,
  TemplateRef,
  ViewChild,
} from "@angular/core";
import {
  DayTemplateContext,
  DayViewModel,
  MonthViewModel,
  NgbDataTemplateStruct,
  NgbDate,
  NgbDatepickerI18n,
  NgbDatepickerService,
  WeekViewModel
} from "@isp/xdce-widget";
import { Subscription } from "rxjs";

@Component({
  selector: 'ngb-fid-datepicker-routing-view',
  host: { class: "d-block" },
  templateUrl: './datepicker-routing-view.html',
  styleUrls: ["./datepicker-routing-view.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class NgbFidDatepickerRoutingView implements OnInit, OnChanges, OnDestroy {
  @Input() dayTemplate: TemplateRef<DayTemplateContext>;
  @Input() monthTemplate: TemplateRef<DayTemplateContext>;
  @Input() yearsTemplate: TemplateRef<DayTemplateContext>;
  @Input() currentTemplate: number;
  @Input() mesi: any;
  @Input() years: any;
  @Input() disabledWeekend: boolean;

  @Input() date: any;
  @Input() dataTemplate: NgbDataTemplateStruct;

  @Input() month: MonthViewModel;

  @Input() outsideDays: "visible" | "hidden" | "collapsed";
  @Input() showWeekdays;
  @Input() showWeekNumbers;

  @Input() startDateEnabled: any;
  @Input() endDateEnabled: any;

  @Input() isToday: boolean;
  @Input() isWeekend: boolean;
  @Input() taxDeadlines: any;

  @Output() select = new EventEmitter<NgbDate>();
  @Output() forceClose = new EventEmitter();
  @Output() changeTemplate = new EventEmitter<NgbDataTemplateStruct>();
  @Output() updateYears = new EventEmitter<number>();

  @ViewChild("ispDatepickerRoutingView") routingView: ElementRef;

  currentYear: number;
  currentMonth: number;
  currentDay: number;
  sub: Subscription;
  tabFocusVisible: boolean;
  shiftTabFromDayActive: boolean = false;

  constructor(
    public i18n: NgbDatepickerI18n,
    private cdr: ChangeDetectorRef,
    private service: NgbDatepickerService,
    private _elementRef: ElementRef
  ) {}
  ngOnInit() {
    // alert("in select in routing : " + this.startDateEnabled)
    // alert("in select in routing : " + this.endDateEnabled)
    // this.currentYear = this.date && this.date.year;
    // this.currentMonth = this.date && this.date.month;
    console.info("routing init", this.currentYear, this.currentMonth);
    this.service.isDayFocusView.subscribe((isDayFocusView) => {
      this.tabFocusVisible = isDayFocusView;
    });
    this.dataTemplate.day = this.date && this.date.day;
    this.dataTemplate.month = this.date && this.date.month;
    this.dataTemplate.year = this.date && this.date.year;
    const model = this.service._model$.getValue();
    if (model && model.selectedDate) {
      this.currentDay = model.selectedDate.day;
      this.currentMonth = model.selectedDate.month;
      this.currentYear = model.selectedDate.year;
      this.selectedDay = model.selectedDate.day;
    }
    this.sub = this.service.model$.subscribe((modelDate) => {
      if (modelDate.focusVisible) {
        if (modelDate && modelDate.focusDate) {
          this.currentDay = modelDate.focusDate.day;
          this.currentMonth = modelDate.focusDate.month;
          this.currentYear = modelDate.focusDate.year;
          if (model && model.selectedDate)
            this.selectedDay = model.selectedDate.day;
        }
        this.cdr.detectChanges();
      }
      this.setFocusDate();
    });
    this.service["subscribeIsDayFocusView"]().subscribe((tabFocusVisible) => {
      this.tabFocusVisible = tabFocusVisible;
      if (tabFocusVisible) {
        this.shiftTabFromDayActive = false;
        this.service.focusVisible = tabFocusVisible;
        this.setFocusDate();
      }
      this.cdr.detectChanges();
    });
  }

  private setFocusDate() {
    let domElement = this._elementRef.nativeElement;
    if (this.currentTemplate == 0 && this.tabFocusVisible) {
      setTimeout(() => {
        const elementToFocus = domElement.querySelector(
          "div.ngb-dp-day > .active"
        );
        if (!!elementToFocus) elementToFocus.focus();
        this.cdr.detectChanges();
      }, 0);
    }
    if (this.currentTemplate == 1 && this.tabFocusVisible) {
      setTimeout(() => {
        const elementToFocus = domElement.querySelector(
          "div.btn-secondary-month.active"
        );
        if (!!elementToFocus) elementToFocus.focus();
        this.cdr.detectChanges();
      }, 50);
    }
    if (this.currentTemplate == 2 && this.tabFocusVisible) {
      setTimeout(() => {
        const elementToFocus = domElement.querySelector(
          "div.btn-secondary-years.active"
        );
        if (!!elementToFocus) elementToFocus.focus();
        this.cdr.detectChanges();
      }, 50);
    }
  }

  isActiveCell(day) {
    if (this.tabFocusVisible) {
      if (!!day.context.focused && day.context.focused) {
        return 0;
      } else if (
        this.service._model$.getValue().selectedDate &&
        day.context.selected
      ) {
        //focus all'apertura
        //caso del giorno preselezionato
        return 0;
      } else if (
        !this.service._model$.getValue().selectedDate &&
        day.context.date.day == this.getToday().day &&
        day.context.date.month == this.getToday().month
      ) {
        //focus all'apertura
        //quando non è preselezionato nessun giorno il focus parte dal giorno corrente
        return 0;
      } else {
        return -1;
      }
    } else {
      if (this.shiftTabFromDayActive) {
        //focus all'apertura
        //caso del giorno preselezionato
        if (
          this.service._model$.getValue().selectedDate &&
          day.context.selected
        ) {
          return 0;
          //quando non è preselezionato nessun giorno il focus parte dal giorno corrente
        } else if (
          !this.service._model$.getValue().selectedDate &&
          day.context.date.day == this.getToday().day &&
          day.context.date.month == this.getToday().month
        ) {
          return 0;
        } else {
          return -1;
        }
      } else {
        return -1;
      }
    }
  }

  onKeyDownTab() {
    //console.info("onKeyDownTab  datepicker routing view")
    this.service.isTabClickedFromDay.next(true);
    this.service.emitIsDayFocusView(false);
  }

  getToday(): NgbDate {
    return new NgbDate(
      new Date().getFullYear(),
      new Date().getMonth() + 1,
      new Date().getDate()
    );
  }

  ngOnDestroy(): void {
    if (this.sub) {
      this.sub.unsubscribe();
    }
  }

  @HostListener("keydown.shift.tab", ["$event"])
  onKeyDown(e) {
    this.service.emitIsDayFocusView(false);
    this.shiftTabFromDayActive = true;
  }
  selectedDay: number = -1;
  ngOnChanges(changes: SimpleChanges) {
    console.info(
      "routing ngOnChanges",
      this.currentYear,
      this.currentMonth,
      this.currentDay
    );
    if (changes.years) {
      this.years = [...changes.years.currentValue];
    }
    if (changes.currentTemplate || changes.dataTemplate) {
      //this.service.isDayFocusView.next(false);
      this.service.emitTemplate(this.currentTemplate);
      this.resetMesi();
      this.resetAnni();
      const model = this.service._model$.getValue();
      if (model && model.focusDate && model.firstDate) {
        //model.focusDate.month = model.firstDate.month;
        //model.focusDate.year = model.firstDate.year;
        //this.service._model$.next(model);
      }
      if (model && model.selectedDate) {
        this.currentDay = model.selectedDate.day;
        this.currentMonth = model.selectedDate.month;
        this.currentYear = model.selectedDate.year;
      }
    }
    if (changes["date"] && changes["date"].currentValue) {
      //l'anno in focus non è compreso nel range della maschera => riscostrisco la maschera
      //se il primo anno della maschera è superiore all'anno selezionato
      if (this.years[0][0].number > this.date.year) {
        this.updateYears.emit(this.years[0][0].number - 12);
        //se l'ultimo anno della maschera è minore dell'anno selezionato
      } else if (this.date.year > this.years[2][3].number) {
        this.updateYears.emit(this.years[2][3].number + 1);
      }
    }
    this.cdr.detectChanges();
  }

  doSelect(day: DayViewModel, $event: any) {
    if (!this.abilitaGiorni(day.date)) {
      return;
    }

    setTimeout(() => this.forceClose.emit(), 0);
    if (this.dataTemplate.valid) {
      // il day si prende quello selezionato e non quello del dataTemplate
      // day.date.day = this.dataTemplate.day;
      day.date.month = this.dataTemplate.month;
      day.date.year = this.dataTemplate.year;
      this.currentYear = day.date.year;
      this.currentMonth = day.date.month;
      this.currentDay = day.date.day;
      this.select.emit(NgbDate.from(day.date));
    } else {
      if (!day.context.disabled && !this.isHidden(day)) {
        this.currentYear = day.date.year;
        this.currentMonth = day.date.month;
        this.currentDay = day.date.day;
        this.select.emit(NgbDate.from(day.date));
      }
    }
    $event.stopPropagation();
  }

  abilitaGiorni(data) {
    let buffer = new Date(data.year, data.month - 1, data.day);

    let startDate = new Date(
      this.startDateEnabled.year,
      this.startDateEnabled.month - 1,
      this.startDateEnabled.day
    );

    let endDate = new Date(
      this.endDateEnabled.year,
      this.endDateEnabled.month - 1,
      this.endDateEnabled.day
    );

    return startDate <= buffer && buffer <= endDate;
  }

  abilitaMesi(data) {
    return (
      ((this.startDateEnabled.year == this.dataTemplate.year &&
        this.startDateEnabled.month <= data.number) ||
        (data.number < this.startDateEnabled.month &&
          this.startDateEnabled.year < this.dataTemplate.year) ||
        (this.startDateEnabled.month <= data.number &&
          this.startDateEnabled.year <= this.dataTemplate.year)) &&
      ((data.number <= this.endDateEnabled.month &&
        this.dataTemplate.year <= this.endDateEnabled.year) ||
        (data.number > this.endDateEnabled.month &&
          this.dataTemplate.year < this.endDateEnabled.year))
    );
  }

  abilitaAnni(data) {
    return (
      this.startDateEnabled.year <= data.number &&
      data.number <= this.endDateEnabled.year
    );
  }

  doSelectMonth(mese: any, $event: any) {
    console.info("doSelectMonth", $event);

    if (!this.abilitaMesi(mese)) {
      return;
    }

    this.currentTemplate = 0;
    this.dataTemplate.template = this.currentTemplate;

    // reset giorni
    this.month.weeks.forEach((week) => {
      week.days.forEach((day) => {
        day.context.selected =
          day.date.day === this.currentDay &&
          day.date.month === this.currentMonth &&
          day.date.year === this.currentYear;
        day.context.focused =
          day.date.day === this.currentDay &&
          day.date.month === this.currentMonth &&
          day.date.year === this.currentYear;
      });
    });
    this.resetMesi();

    this.dataTemplate.month = mese.number;
    this.dataTemplate.valid = true;
    this.changeTemplate.emit(this.dataTemplate);
    $event.stopPropagation();
  }

  doSelectYear(year: any, $event: any) {
    if (!this.abilitaAnni(year)) {
      return;
    }

    this.currentTemplate = 1;
    this.dataTemplate.template = this.currentTemplate;

    // resetta mesi e anno
    this.resetMesi();

    this.dataTemplate.year = year.number;
    this.dataTemplate.valid = true;
    this.changeTemplate.emit(this.dataTemplate);
    $event.stopPropagation();
  }

  isCollapsed(week: WeekViewModel) {
    return (
      this.outsideDays === "collapsed" &&
      week.days[0].date.month !== this.month.number &&
      week.days[week.days.length - 1].date.month !== this.month.number
    );
  }

  isHiddenWeek(week: WeekViewModel) {
    let counter = 0;
    for (let i = 0; i < week.days.length; i++) {
      if (this.isHidden(week.days[i])) {
        counter++;
      }
    }
    return counter === 7;
  }

  isHidden(day: DayViewModel) {
    return (
      this.outsideDays === "hidden" && this.month.number !== day.date.month
    );
  }

  getRoutingViewHeight() {
    return this.routingView.nativeElement.offsetHeight;
  }

  getRoutingViewWidth() {
    return this.routingView.nativeElement.offsetWidth;
  }

  private resetMesi() {
    let months: [][] = [];
    months = this.mesi.map((row) => {
      return row.map((m) => {
        const res = { ...m };
        res.selected =
          res.number === this.currentMonth &&
          this.dataTemplate.year === this.currentYear;
        res.focused =
          res.number === this.currentMonth &&
          this.dataTemplate.year === this.currentYear;
        return res;
      });
    });
    this.mesi = months;
  }

  private resetAnni() {
    let yrs: [][] = [];
    yrs = this.years.map((row) => {
      return row.map((y) => {
        const res = { ...y };
        res.selected = res.number === this.currentYear;
        res.focused = res.number === this.currentYear;
        return res;
      });
    });
    this.years = yrs;
  }
}
