import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  EventEmitter,
  forwardRef,
  HostBinding,
  Inject,
  InjectFlags,
  Injector,
  Input,
  OnDestroy,
  Optional,
  Output,
  SimpleChanges,
  ViewChild,
} from "@angular/core";
import {
  NG_ASYNC_VALIDATORS,
  NG_VALIDATORS,
  NG_VALUE_ACCESSOR,
  NgControl,
  NgModel,
} from "@angular/forms";
import {
  APPLICATION_CONTEXT_MANAGER,
  ApplicationContextManagerInterface,
  EventManager
} from "@isp/xdce-arch-core-base";
import {
  AsyncValidatorArray, coerceBooleanProperty, DateValidator, EnabledRangeService, FormTouchService,
  I18nNOA11Y,
  INbpInput, NbpCalendarPattern,
  NbpCalendarPosition, NbpDatepickerConfigurationNOA11Y,
  NbpDatepickerItI18nNOA11Y, NbpDateSeparator, NbpFormContainerComponent,
  NbpInputElementBase,
  NbpItDateParserFormatterNOA11Y,
  NbpStyle, NgbDate, NgbDateParserFormatterNOA11Y, NgbDatepickerI18nNOA11Y, NgbDateStruct, ValidatorArray,
  WidgetMode
} from "@isp/xdce-widget";
import { Observable } from "rxjs";
import { NgbFidInputDatepickerNOA11Y } from "./@ng-bootstrap-noa11y/datepicker/datepicker-input";


/**
 * Provider Expression that allows nbp-calendar-generic to register as a ControlValueAccessor.
 * This allows it to support [(ngModel)].
 * @docs-private
 */
export const NBP_CALENDAR_CONTROL_VALUE_ACCESSOR_NOA11Y: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => NbpCalendarGenericComponentNOA11Y), // un riferimento al futuro componente corrente
  multi: true,
};

export const NBP_INPUT_CALENDAR_ELEMENT_BASE_NOA11Y: any = {
  provide: NbpInputElementBase,
  useExisting: forwardRef(() => NbpCalendarGenericComponentNOA11Y), // un riferimento al futuro componente corrente
  multi: true,
};

/** intero autogenerato usato per id unici per nbpCombo component */
let nextId = 0;

@Component({
  templateUrl: './nbp-calendar-generic.component.noa11y.html',
  styleUrls: ["./nbp-calendar-generic.component.noa11y.scss"],
  providers: [
    EnabledRangeService,
    I18nNOA11Y,
    NbpDatepickerConfigurationNOA11Y,
    { provide: NgbDatepickerI18nNOA11Y, useClass: NbpDatepickerItI18nNOA11Y },
    {
      provide: NgbDateParserFormatterNOA11Y,
      useClass: NbpItDateParserFormatterNOA11Y,
    },
    NBP_CALENDAR_CONTROL_VALUE_ACCESSOR_NOA11Y,
    NBP_INPUT_CALENDAR_ELEMENT_BASE_NOA11Y,
  ],
  host: {
    "(blur)": "touchInput()",
  },
  changeDetection: ChangeDetectionStrategy.OnPush,
  //  host: {
  //    '(document:click)': 'focusOutComponent($event)'
  //  }
})
export class NbpCalendarGenericComponentNOA11Y
  extends NbpInputElementBase<any>
  implements INbpInput, AfterViewInit, OnDestroy {
  /**
   * (opzionale ) Boolean for keeping the value of the date
   * when the input is disabled
   */
  @Input() keepDateOnDisabled: boolean = false;
  @Input() nbpModelOptions: {
    name?: string;
    standalone?: boolean;
    updateOn?: "change" | "blur" | "submit";
  };

  /**
   * (opzionale ) Booleano che gestisce  l’attributo  bordo giallo del campo di input.
   * Al primo click/focus tornerà alla versione di default.
   * valore default false
   */
  @Input() nbpHighlightedDefault: boolean = false;
  /**
   * (Opzionale)
   * Attributo HTML standard: definisce il pattern di validazione del campo di input
   */
  @Input() pattern: RegExp;
  /**
   * Nome del campo di input associato al calendar.
   */
  @Input() name: string;

  @Input() nbpDarkBackground: boolean = false;

  /**
   * Flag che identifica se la textarea è in modalità Output: Default false
   */
  _nbpOutMode: boolean = false;
  @Input()
  get nbpOutput(): any {
    return this._nbpOutMode;
  }
  set nbpOutput(value) {
    this._nbpOutMode = coerceBooleanProperty(value);
  }

  /**
   * Identificativo univoco per il text field.
   * Se nessun id è fornito e' auto-generato.
   */
  @Input() id = `nbp-calendar-generic-${++nextId}`;

  hasFocus: boolean = false;

  _nbpShowValidation: boolean;
  /**
   * (Opzionale)
   * Booleano che consente di visualizzare i messaggi di feedback relativi alla validazione.
   * Valore di default false
   */
  @HostBinding("class.nbp-show-validation")
  @Input()
  get nbpShowValidation(): boolean {
    if (this.parentFormContainer) {
      this._nbpShowValidation = this.parentFormContainer.nbpShowValidation;
    }
    return this._nbpShowValidation;
  }

  set nbpShowValidation(value) {
    this._nbpShowValidation = coerceBooleanProperty(value);
  }

  /**
   * testo del placeholder
   * Valore di Default: "dd.mm.yyyy"
   */
  @Input() placeholder = "";

  /**
   * (Opzionale) Messaggio di errore da visualizzare in caso di campo non valido
   * Valore di Default: "Cmpo obbligatorio o invalido"
   */
  @Input() nbpErrorMessage: string;

  @Input() showErrorLabel = true;

  ngbMaxDate: NgbDateStruct = { day: 31, month: 12, year: 2050 };
  nbpMaxDate: Date = new Date(
    this.ngbMaxDate.year,
    this.ngbMaxDate.month - 1,
    this.ngbMaxDate.day
  );

  /**
   * La massima data accettata dal componente e visualizzata nel calendario
   * Valore di Default: 31.12.2050
   */
  @Input()
  get maxDate() {
    return this.nbpMaxDate;
  }
  set maxDate(maxDate: Date) {
    this.nbpMaxDate = maxDate;
    this.ngbMaxDate = this.parseDateStructFromDate(maxDate) as NgbDateStruct;
    if (this.model && this.model.value) {
      this.touchInput();
      this.propagateNgClassToInnerInput();
    }
  }

  ngbMinDate: NgbDateStruct = { day: 1, month: 1, year: 1000 };
  nbpMinDate: Date = new Date(
    this.ngbMinDate.year,
    this.ngbMinDate.month - 1,
    this.ngbMinDate.day
  );

  /**
   * La minima data accettata dal componente e visualizzata nel calendario
   * Valore di Default: 01.01.1970
   */
  @Input()
  get minDate() {
    return this.nbpMinDate;
  }
  set minDate(minDate: Date) {
    this.nbpMinDate = minDate;
    this.ngbMinDate = this.parseDateStructFromDate(minDate) as NgbDateStruct;
    if (this.model && this.model.value) {
      this.touchInput();
      this.propagateNgClassToInnerInput();
    }
  }

  // @Input() startDateEnabled: { year: number, month: number, day: number };
  // @Input() endDateEnabled: { year: number, month: number, day: number };

  _startDateEnabled: any;
  /**
   * Data di inizio delle date abilitate/selezionabili.
   */
  @Input()
  get startDateEnabled(): any {
    return this._startDateEnabled;
  }

  set startDateEnabled(value) {
    this._startDateEnabled = value ? value : this.ngbMinDate;
    this.minDate = this._startDateEnabled;
  }

  // -----------

  _endDateEnabled: any;
  /**
   * Data di fine delle date abilitate/selezionabili.
   */
  @Input()
  get endDateEnabled(): any {
    return this._endDateEnabled;
  }

  set endDateEnabled(value) {
    this._endDateEnabled = value ? value : this.ngbMaxDate;
    this.maxDate = this._endDateEnabled;
  }

  protected _nbpDisabled: boolean;
  /**
   * Flag che abilita/disabilita le funzionalità associate al calendar
   */
  @Input()
  get disabled(): boolean {
    if (this.parentFormContainer && this.parentFormContainer.disabled) {
      this._nbpDisabled = true;
    }
    return this._nbpDisabled;
  }
  set disabled(value) {
    this._nbpDisabled = coerceBooleanProperty(value);
    if (this._nbpDisabled) {
      this.rplCalendar.close();
      this.emitFocus(false);
    }
    if (!this.keepDateOnDisabled) {
      this._ngbModelChanged(null);
    }
  }

  private _nbpRequired: boolean;
  /**
   * Flag che imposta come obbligatorio o meno il campo di input (disabilitato) associato al calendar
   */
  @Input()
  get required(): boolean {
    return this._nbpRequired;
  }
  set required(value) {
    this._nbpRequired = coerceBooleanProperty(value);
  }

  @Output() nbpOnNavigate: EventEmitter<{ current: any; next: any }> =
    new EventEmitter<{ current: any; next: any }>();
  @Output() modelChange: EventEmitter<any> = new EventEmitter<any>();
  @Output() focus: EventEmitter<boolean> = new EventEmitter();

  invalid: Observable<boolean>;

  ngControl: NgControl;
  // dirty: boolean = false;

  /**
   * (Opzionale) Enum per impostare lo stile del pulsante.
   * "DEFAULT" | "PRIMARY" | "SECONDRY"
   *  Valore default: "DEFAULT"
   */
  @Input() nbpStyle: NbpStyle = NbpStyle.DEFAULT;

  _style: any = NbpStyle;

  /*
   * Booleano che identifica se il dispositivo è mobile o no
   */
  isNative: boolean;

  /*
   * Identificativo che associa il tipo alla input (string o date)
   */
  private __type: string;

  @Input()
  set _type(s: string) {
    this.__type = s;
  }

  get _type(): string {
    if (!this.__type) {
      return !this.value ? "text" : "date";
    } else {
      return this.__type;
    }
  }

  /*
   * Booleano che disabilita la possibilità di selezionare i weekend
   */
  public _disableWeekend: boolean = false;
  @Input()
  get disabledWeekend(): boolean {
    return this._disableWeekend;
  }
  set disabledWeekend(value) {
    this._disableWeekend = coerceBooleanProperty(value);
  }

  /*
   * Lista di giorni da disabilitare
   */
  public _holidays: Array<NgbDateStruct>;
  @Input()
  get holidays(): any {
    return this._holidays;
  }
  set holidays(value) {
    this._holidays = value ? value : this._holidays;
  }

  /**
   * Lista delle scadenze fiscali
   */
  public _tax: Array<NgbDateStruct>;
  @Input()
  get taxDeadlines(): any {
    return this._tax;
  }
  set taxDeadlines(value) {
    this._tax = value ? value : this._tax;
  }

  @ViewChild("rplCalendar", { read: NgbFidInputDatepickerNOA11Y })
  rplCalendar: NgbFidInputDatepickerNOA11Y;
  @ViewChild(NgModel) model: NgModel;

  /**
   * (Opzionale) Paramentro da valorizzare per abilitare l'autofocus sul rispettivo campo.
   */
  @Input() firstFocus: boolean = false;

  /**
   * Parametro usato per attivare e disattivare il focus al tab.
   */
  @Input() focusOn: boolean;

  /**
   * (Optional) Input parameter used to set the tabindex on the component.
   */
  @Input() tabindex = 0;
  /*------------------------------------------------------------------------------------------------------------------*/

  /**
   * (Optional) Input parameter used to set the label of aria-labelledby.
   */
  @Input() label: string = null;
  /*------------------------------------------------------------------------------------------------------------------*/

  /**
   * (Optional) Input parameter used to set the label of aria-label.
   */
  @Input() ariaLabel: string = null;
  /*------------------------------------------------------------------------------------------------------------------*/

  @Input() nbpLightMode: boolean = false;

  private _pattern: any = NbpCalendarPattern;
  @Input() nbpLabelPattern: NbpCalendarPattern = this._pattern.GGMMAAAA;

  private _separator = NbpDateSeparator;
  @Input() nbpDateSeparator: NbpDateSeparator = this._separator.DOT;

  /**
   * (Opzionale) booleano per datepicker aperto di default
   */
  @Input() opened: boolean;

  private _placement: any = NbpCalendarPosition;
  /**
   * (Opzionale) NbpCalendarPosition per determinare il posizionamento del datepicker all'apertura
   * default NbpCalendarPosition.BOTTOM_LEFT
   */
  @Input() nbpPlacement: string = this._placement.BOTTOM_LEFT;

  /**
   * Input parameter used to enable the default view on UNIFIEDVG widgetmode (also on native device).
   */
  @Input() enableDefaultViewOnUnifiedVG: boolean = false;

  /**
   * (Opzionale) Boolean che definisce se la posizione del calendario va gestita in maniera dinamica:
   * Default: "FALSE"
   */
  @Input() enableDynamicPosition: boolean = false;

  @ViewChild("autoFocusCalendar", { read: ElementRef })
  private elementRef: ElementRef;
  @ViewChild("autoFocusCalendar", { read: NgModel })
  private innerNgModel: NgModel;

  _ngbModel: any;
  //private mutationobs;
  error_message_visible: boolean = false;

  private _hostNgModel: NgModel;
  get hostNgModel(): NgModel {
    if (!this._hostNgModel) {
      this._hostNgModel = this.inj.get(NgModel, null, InjectFlags.Host);
    }
    return this._hostNgModel;
  }

  //FIX CALENDAR NULL VALUE
  set value(value: any) {
    if (
      this.isNative &&
      value &&
      (<string>value).split &&
      (<string>value).split("-").length === 3
    ) {
      const [year, month, day] = (<string>value)
        .split("-")
        .map((x) => parseInt(x));
      value = new NgbDate(year, month, day);
    }
    if (this.innerValue !== value) {
      this.innerValue = value;
      if (!value) {
        this.elementRef.nativeElement.value = null;
      }
      this.changed.forEach((f) => f(value));
    }
  }

  private get isInMinicombo(): boolean {
    let parent = (<HTMLElement>this.er.nativeElement).parentElement;
    while (
      !parent.tagName.match(/minicombo/i) ||
      !!parent.tagName.match(/body/i)
    ) {
      parent = parent.parentElement;
    }
    return !!parent.tagName.match(/minicombo/i);
  }

  constructor(
    private enabledRange: EnabledRangeService,
    private eventManager: EventManager,
    @Optional() @Inject(NG_VALIDATORS) private validatori: ValidatorArray,
    @Optional()
    @Inject(NG_ASYNC_VALIDATORS)
    private asyncValidatori: AsyncValidatorArray,
    @Optional() private parentFormContainer: NbpFormContainerComponent,
    private element: ElementRef,
    private inj: Injector,
    @Optional() private formTouchService: FormTouchService,
    private changeDetectorRef: ChangeDetectorRef,
    @Inject(APPLICATION_CONTEXT_MANAGER)
    private applicationContext: ApplicationContextManagerInterface
  ) {
    super(validatori, asyncValidatori, inj); // ValueAccessor base
    if (formTouchService) {
      formTouchService.registerOnTouch(() => {
        this.touchInput();
      });
    }

    if (this.opened) {
      if (!this.isNative) {
        this.rplCalendar.toggle(this.corporateMode);
        this.changeDetectorRef.detectChanges();
      } else {
        this.element.nativeElement.querySelector("input").focus();
      }
    }

    this.eventManager.subscribeToEvent("HANDLER_CALENDAR", (payload) => {
      if (payload && payload.id !== this.id && this.rplCalendar.isOpen()) {
        this.rplCalendar.close();
        this.emitFocus(false);
      }
    });

    if (this.unifiedMode) {
      this.eventManager.subscribeToEvent("DATEPICKER_CLOSE", () =>
        this.emitFocus(false)
      );
    }
  }

  _onTouchedEvent(event: any) {
    this.touch();
    this.showErrorLabel = true;
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.taxDeadlines) {
      this._tax = changes.taxDeadlines.currentValue;
    }
  }

  public getIdAutonegerato() {
    return this.id;
  }

  private propagateNgClassToInnerInput() {
    const callback = (mutationsList, observer) => {
      mutationsList
        .filter((m) => m.type === "attributes" && m.attributeName === "class")
        .forEach((m) => {
          if (
            m.target.classList.contains("ng-touched") &&
            m.oldValue.trim().match(/ng-untouched/)
          ) {
            this.innerNgModel.control.markAsTouched();
          } else if (
            m.target.classList.contains("ng-untouched") &&
            m.oldValue.trim().match(/ng-touched/)
          ) {
            this.innerNgModel.control.markAsUntouched();
          }

          if (
            m.target.classList.contains("ng-dirty") &&
            m.oldValue.trim().match(/ng-pristine/)
          ) {
            this.innerNgModel.control.markAsDirty();
          } else if (
            m.target.classList.contains("ng-pristine") &&
            m.oldValue.trim().match(/ng-dirty/)
          ) {
            this.innerNgModel.control.markAsPristine();
          }
        });
    };
    const mutationobs = new MutationObserver(callback);
    mutationobs.observe(this.element.nativeElement, {
      attributes: true,
      attributeOldValue: true,
    });
    return mutationobs;
  }

  ngOnInit() {
    super.ngOnInit();

    //this.mutationobs = this.propagateNgClassToInnerInput();

    this.registerOnTouched(this.touchInput.bind(this));

    let elemento: HTMLElement = this.element.nativeElement;

    const options = {
      weekday: "long",
      year: "numeric",
      month: "long",
      day: "numeric",
    };

    this.ngControl = this.inj.get(NgControl);

    this.isNative =
      /(ip(a|o)d|iphone|android)/gi.test(window.navigator.userAgent) &&
      this.widgetMode !== WidgetMode.CORPORATE;

    window.sessionStorage.setItem("isNative", JSON.stringify(this.isNative));

    this._type = this.isNative ? "date" : "text";

    this.enabledRange.startDate = this.ngbMinDate;
    this.enabledRange.endDate = this.ngbMaxDate;
    if (this.startDateEnabled) {
      this.enabledRange.startDate = this.startDateEnabled;
    }

    if (this.endDateEnabled) {
      this.enabledRange.endDate = this.endDateEnabled;
    }

    if (this.firstFocus) {
      this.elementRef.nativeElement.focus();
      this.elementRef.nativeElement.click();

      // this._focusActive = true;
    }

    if (this.opened) {
      this.rplCalendar.opened = true;
    }

    //used to display errors on form async validations
    this.propagateNgClassToInnerInput();
  }

  get retailMode() {
    return this.widgetMode === WidgetMode.RETAIL;
  }

  get corporateMode() {
    return this.widgetMode === WidgetMode.CORPORATE;
  }

  get unifiedMode() {
    return (
      this.widgetMode === WidgetMode.UNIFIEDVG ||
      this.widgetMode === WidgetMode.ISYBANKM
    );
  }

  touchInput() {
    this.model.control.markAsTouched();
    this.model.control.markAsDirty();
    this.element.nativeElement.click();
    this.showErrorLabel = true;
    // this._onTouchedEvent(null);
  }

  validaData(data) {
    let buffer = new Date(data.year, data.month - 1, data.day);
    let startDate = new Date(
      this._startDateEnabled.year,
      this._startDateEnabled.month - 1,
      this._startDateEnabled.day
    );
    let endDate = new Date(
      this._endDateEnabled.year,
      this._endDateEnabled.month - 1,
      this._endDateEnabled.day
    );
    if (startDate <= buffer && buffer <= endDate) {
      return true;
    }
    return false;
  }

  _ngbModelChanged(event: NgbDateStruct) {
    this.error_message_visible = false;

    // if(!this.validaData(event)){
    //   return this._ngbModel.invalid = true;
    // }

    if (!event) {
      if (this.unifiedMode && this.isNative) {
        // this._type = 'text';
      }
      this.error_message_visible = true;
      this.elementRef.nativeElement.value = null;
      this.rplCalendar.writeValue(event);
      this._ngbModel = null;
      this.value = null;
    } else {
      this.error_message_visible = true;
      if (typeof event === "string") {
        this.renderer.setProperty(
          this.elementRef.nativeElement,
          "value",
          event
        );
      } else {
        this.error_message_visible = false;
        this.rplCalendar.writeValue(event);
      }
      this.value = this.parseDateFromDateStruct(event);
      this.setAriaLabel(event);
    }
    this.changeDetectorRef.detectChanges();
    this.modelChange.emit(event);
    this.emitHandlerCalendar({ id: this.id });
  }

  touchExternalInput() {
    if (this.hostNgModel && this.hostNgModel.control) {
      this.hostNgModel.control.markAsDirty({ onlySelf: true });
      this.hostNgModel.control.markAsTouched({ onlySelf: true });
    }
  }

  private parseDateFromDateStruct(value: NgbDateStruct) {
    return value && value instanceof Date
      ? new Date(value.year, value.month - 1, value.day)
      : value;
  }

  private parseDateStructFromDate(value: Date) {
    return value && value instanceof Date
      ? {
        day: value.getDate(),
        month: value.getMonth() + 1,
        year: value.getFullYear(),
      }
      : value;
  }

  writeValue(value: Date) {
    this._ngbModel = this.parseDateStructFromDate(value);
    if (!value && this.unifiedMode && this.isNative) {
      // this._type = 'text';
      if (this.model && this.model.control) {
        this.model.control.setValue(null);
      }
    }
    this.value = value;
    this.changeDetectorRef.detectChanges();
    // !value && this.unifiedMode && this.isNative && this.model && this.model.control && this.model.control.setValue(null)
  }

  _onNavigate(navigateEvent: { current: NgbDateStruct; next: NgbDateStruct }) {
    let _current: any = navigateEvent.current
      ? new Date(navigateEvent.current.year, navigateEvent.current.month - 1, 1)
      : navigateEvent.current;
    let _next: any = navigateEvent.next.year
      ? new Date(navigateEvent.next.year, navigateEvent.next.month - 1, 1)
      : navigateEvent.next.year;

    this.changeDetectorRef.detectChanges();
    this.nbpOnNavigate.emit({ current: _current, next: _next });
  }

  private emitHandlerCalendar(payload) {
    this.eventManager.emit("HANDLER_CALENDAR", payload);
  }

  public handleCalendar($event) {
    if (!this.disabled) {
      const payload = { id: this.id, isNative: false };

      if (!this.isNative || this.enableDefaultViewOnUnifiedVG) {
        if (!this.opened) {
          if (this._hostNgModel && this._hostNgModel.model == null) {
            this.rplCalendar.writeValue(null);
            this.rplCalendar.startDate = null;
            let today = new Date();
            let todayObject = {
              year: today.getFullYear(),
              month: today.getMonth() + 1,
              day: today.getDate(),
            };
            if (
              this.endDateEnabled != null &&
              this.dateAbiggerThanDateB(todayObject, this.endDateEnabled)
            ) {
              this.rplCalendar.startDate = this.endDateEnabled;
            }
            if (
              this.startDateEnabled != null &&
              this.dateAbiggerThanDateB(this.startDateEnabled, todayObject)
            ) {
              this.rplCalendar.startDate = this.startDateEnabled;
            }
          } else if (this._hostNgModel && this._hostNgModel.model != null) {
            let dateParsed = this.parseDateStructFromDate(
              this._hostNgModel.model
            );
            let startDateParsed: NgbDateStruct;
            if (dateParsed) {
              dateParsed instanceof Date
                ? (startDateParsed = null)
                : (startDateParsed = dateParsed);
            }
            this.rplCalendar.startDate = startDateParsed;
            if (
              this.endDateEnabled != null &&
              this.dateAbiggerThanDateB(startDateParsed, this.endDateEnabled)
            ) {
              this.rplCalendar.startDate = this.endDateEnabled;
            }
            if (
              this.startDateEnabled != null &&
              this.dateAbiggerThanDateB(this.startDateEnabled, startDateParsed)
            ) {
              this.rplCalendar.startDate = this.startDateEnabled;
            }
          }
          if (this.elementRef && this.elementRef.nativeElement)
            this.elementRef.nativeElement.focus();
          this.rplCalendar.toggle(this.corporateMode);
        } else {
          this.rplCalendar.open(this.corporateMode);
        }
        this.changeDetectorRef.detectChanges();
      } else {
        payload.isNative = true;
        this._type = "date";
        setTimeout(() => {
          this.element.nativeElement.querySelector("input").focus();
          this.element.nativeElement.querySelector("input").click();
        }, 0);
      }
      $event.stopPropagation();
      if (window.event) {
        if (window.event.stopPropagation) {
          window.event.stopPropagation();
        } else {
          window.event.cancelBubble = true;
        }
      }

      this.emitHandlerCalendar(payload);
    }
    if (this.applicationContext && this.applicationContext.applicationContext) {
      this.applicationContext.applicationContext.userInteraction = true;
    }
    this.emitFocus(this.rplCalendar.isOpen());
  }

  nativeClickHandler() {
    if (
      this.isNative &&
      this.unifiedMode &&
      !this.enableDefaultViewOnUnifiedVG
    ) {
      this._type = "date";
      this.element.nativeElement.querySelector("input").focus();
      this.element.nativeElement.querySelector("input").click();
    }
  }

  /**
   * @function setAriaLabel() - Set the aria-label of the calendar.
   * @param {any} date
   */
  public setAriaLabel(date: any) {
    let localeString: string;
    switch (this.nbpLabelPattern) {
      case this._pattern.MMGGAAAA:
        localeString = "en-US";
        break;
      case this._pattern.AAAAMMGG:
        localeString = "hu-HU";
        break;
      default:
        localeString = "en-GB";
    }
    if (date) {
      if (typeof date === "string") {
        if (this._pattern.MMAAAA) {
          this.ariaLabel = "01." + date;
        }
      } else if (date instanceof Date) {
        this.ariaLabel = date.toLocaleDateString(localeString);
      } else {
        if (this._pattern.MMAAAA) {
          this.ariaLabel = new Date(
            date.year,
            date.month - 1,
            1
          ).toLocaleDateString(localeString);
        } else {
          this.ariaLabel = new Date(
            date.year,
            date.month - 1,
            date.day
          ).toLocaleDateString(localeString);
        }
      }
    } else {
      this.ariaLabel = new Date().toLocaleDateString(localeString);
    }
    return this.ariaLabel;
  }

  public setSpanLabel(date) {
    const options = { year: "numeric", month: "2-digit", day: "2-digit" };

    if (date) {
      if (date instanceof Date) {
        this.ariaLabel = date.toLocaleDateString("de-DE", options);
      } else {
        this.ariaLabel = new Date(
          date.year,
          date.month - 1,
          date.day
        ).toLocaleDateString("de-DE", options);
      }
    } else {
      this.ariaLabel = new Date().toLocaleDateString("de-DE", options);
    }

    let dateArray = this.ariaLabel.split(".");

    switch (this.nbpLabelPattern) {
      case this._pattern.MMGGAAAA:
        this.ariaLabel = `${dateArray[1]}${this.nbpDateSeparator}${dateArray[0]}${this.nbpDateSeparator}${dateArray[2]}`;
        break;
      case this._pattern.AAAAMMGG:
        this.ariaLabel = `${dateArray[2]}${this.nbpDateSeparator}${dateArray[1]}${this.nbpDateSeparator}${dateArray[0]}`;
        break;
      default:
        this.ariaLabel = `${dateArray[0]}${this.nbpDateSeparator}${dateArray[1]}${this.nbpDateSeparator}${dateArray[2]}`;
    }

    return this.ariaLabel;
  }

  public focusOutComponent(event: any) {
    if (this.elementRef.nativeElement.contains(event.target)) {
      if (!this.rplCalendar.isOpen()) {
        this.rplCalendar.open(this.corporateMode);
      }
    } else {
      if (this.rplCalendar.isOpen()) {
        this.rplCalendar.close();
        this.emitFocus(this.rplCalendar.isOpen());
      }
    }
    this.changeDetectorRef.detectChanges();
    //event.stopPropagation();
  }

  onFocusIn() {
    this.hasFocus = true;
    this.emitFocus();
  }

  onFocusOut() {
    this.hasFocus = false;
    this.emitFocus();
  }

  private emitFocus(hasFocus?: boolean): void {
    if (this.unifiedMode) {
      this.focus.emit(hasFocus === undefined ? this.hasFocus : hasFocus);
    }
  }

  ngAfterViewInit() {
    if (this.corporateMode) {
      this.isNative = this.isNative && !this.isInMinicombo;
    }

    const dateValidator = new DateValidator(this.inj);

    if (!this.value && this.unifiedMode && this.isNative) {
      // this._type = 'text';
    }

    if (dateValidator) {
      if (this.hostNgModel && this.hostNgModel.control) {
        this.hostNgModel.control.setValidators(
          dateValidator.validate.bind(dateValidator)
        );
      }
    }

    let initialModel = this._hostNgModel && this._hostNgModel.model;

    if (initialModel != undefined) {
      this.setAriaLabel(initialModel);
    }

    setTimeout(() => this.changeDetectorRef.detectChanges());
  }

  ngOnDestroy() {
    /* if (this.mutationobs && typeof this.mutationobs.disconnect === 'function') {
      this.mutationobs.disconnect();
    } */
  }
  getCalendarHeight() {
    return this.rplCalendar.getCalendarHeight();
  }

  getCalendarWidth() {
    return this.rplCalendar.getCalendarWidth();
  }

  dateAbiggerThanDateB(a, b) {
    let dateA = new Date(a.year, a.month, a.day);
    let dateB = new Date(b.year, b.month, b.day);

    if (dateA > dateB) {
      return true;
    }
    return false;
  }
}
