

import { AfterViewInit, ChangeDetectorRef, Component, ElementRef, EventEmitter, HostBinding, Inject, Injector, Input, Optional, Output, SimpleChanges, Type, forwardRef } from '@angular/core';
import { NG_ASYNC_VALIDATORS, NG_VALIDATORS, NG_VALUE_ACCESSOR } from '@angular/forms';

import { AsyncValidatorArray, FormTouchService, NbpFormContainerComponent, NbpInputElementBase, NbpInputElementBaseComponentA11YWrapper, NbpStyle, ValidatorArray } from '@isp/xdce-widget/base';

import { APPLICATION_CONTEXT_MANAGER, ApplicationContextManagerInterface, EventManager } from '@isp/xdce-arch-core-base';
import { EnabledRangeService, NbpCalendarPattern, NbpDateSeparator, NgbDateStruct } from '@isp/xdce-widget';
import { NbpCalendarGenericComponentA11Y } from './nbp-calendar-generic.component.a11y';
import { NbpCalendarGenericComponentNOA11Y } from './nbp-calendar-generic.component.noa11y';



export const NBP_CALENDAR_CONTROL_VALUE_ACCESSOR: any = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => NbpFidCalendarGeneric), // un riferimento al futuro componente corrente
    multi: true
};

export const NBP_INPUT_CALENDAR_ELEMENT_BASE: any = {
    provide: NbpInputElementBase,
    useExisting: forwardRef(() => NbpFidCalendarGeneric), // un riferimento al futuro componente corrente
    multi: true
};

@Component({
    selector : "nbp-fid-calendar-generic",
    template: "<ng-container #componentContainer></ng-container>",
    providers: [
        NBP_CALENDAR_CONTROL_VALUE_ACCESSOR,
        NBP_INPUT_CALENDAR_ELEMENT_BASE,
    ]
})
export class NbpFidCalendarGeneric extends NbpInputElementBaseComponentA11YWrapper<NbpCalendarGenericComponentA11Y, NbpCalendarGenericComponentNOA11Y, any> implements AfterViewInit {

    constructor(enabledRange: EnabledRangeService,
        eventManager: EventManager,
        @Optional() @Inject(NG_VALIDATORS) validatori: ValidatorArray,
        @Optional() @Inject(NG_ASYNC_VALIDATORS) asyncValidatori: AsyncValidatorArray,
        @Optional() parentFormContainer: NbpFormContainerComponent,
        element: ElementRef,
        inj: Injector,
        @Optional() formTouchService: FormTouchService,
        private changeDetectorRef: ChangeDetectorRef,
        @Inject(APPLICATION_CONTEXT_MANAGER) applicationContext: ApplicationContextManagerInterface) {
        super(validatori, asyncValidatori, inj);
    }
  ngAfterViewInit(): void {
    this.model.valueChanges.subscribe(() => this.changeDetectorRef.detectChanges());
  }

    get A11Ycomponent(): Type<NbpCalendarGenericComponentA11Y> {
        return NbpCalendarGenericComponentA11Y;
    }

    get noA11Ycomponent(): Type<NbpCalendarGenericComponentNOA11Y> {
        return NbpCalendarGenericComponentNOA11Y;
    }

    protected modelFromContentChild: boolean = false;

    ngOnInit(): void { 
        super.ngOnInit();
        this.model = this.component.model;
    }

    private _keepDateOnDisabled: boolean = undefined;
    @Input()
    set keepDateOnDisabled(value: boolean) {
        this._keepDateOnDisabled = value;
        if (this.component) {
            this.component.keepDateOnDisabled = value;
        }
    }
    get keepDateOnDisabled(): boolean {
        return this._keepDateOnDisabled;
    }


    private _nbpModelOptions: {
        name?: string,
        standalone?: boolean,
        updateOn?: 'change'|'blur'|'submit'
      } = undefined;
    @Input()
    set nbpModelOptions(value: {
        name?: string,
        standalone?: boolean,
        updateOn?: 'change'|'blur'|'submit'
      }) {
        this._nbpModelOptions = value;
        if (this.component) {
            this.component.nbpModelOptions = value;
        }
    }
    get nbpModelOptions(): {
        name?: string,
        standalone?: boolean,
        updateOn?: 'change'|'blur'|'submit'
      } {
        return this._nbpModelOptions;
    }


    private _nbpHighlightedDefault: boolean = undefined;
    @Input()
    set nbpHighlightedDefault(value: boolean) {
        this._nbpHighlightedDefault = value;
        if (this.component) {
            this.component.nbpHighlightedDefault = value;
        }
    }
    get nbpHighlightedDefault(): boolean {
        return this._nbpHighlightedDefault;
    }


    private _pattern: RegExp = undefined;
    @Input()
    set pattern(value: RegExp) {
        this._pattern = value;
        if (this.component) {
            this.component.pattern = value;
        }
    }
    get pattern(): RegExp {
        return this._pattern;
    }


    private _name: string = undefined;
    @Input()
    @HostBinding('attr.name')
    set name(value: string) {
        this._name = value;
        if (this.component) {
            this.component.name = value;
        }
    }
    get name(): string {
        return this._name;
    }


    private _nbpDarkBackground: boolean = undefined;
    @Input()
    set nbpDarkBackground(value: boolean) {
        this._nbpDarkBackground = value;
        if (this.component) {
            this.component.nbpDarkBackground = value;
        }
    }
    get nbpDarkBackground(): boolean {
        return this._nbpDarkBackground;
    }


    private _nbpOutput: boolean = undefined;
    @Input()
    set nbpOutput(value: boolean) {
        this._nbpOutput = value;
        if (this.component) {
            this.component.nbpOutput = value;
        }
    }
    get nbpOutput(): boolean {
        return this._nbpOutput;
    }


    private _id: string = undefined;
    @Input()
    set id(value: string) {
        this._id = value;
        if (this.component) {
            this.component.id = value;
        }
    }

    get id(): string {
      if(this.component && this._id == undefined){
        this._id = this.component.getIdAutonegerato();
      }
      return this._id;
    }

    private _nbpShowValidation: boolean= undefined;
    @Input()
    set nbpShowValidation(value: boolean) {
        this._nbpShowValidation = value;
        if (this.component) {
            this.component.nbpShowValidation = value;
        }
    }
    get nbpShowValidation(): boolean {
        return this._nbpShowValidation;
    }


    private _placeholder: string = undefined;
    @Input()
    set placeholder(value: string) {
        this._placeholder = value;
        if (this.component) {
            this.component.placeholder = value;
        }
    }
    get placeholder(): string {
        return this._placeholder;
    }


    private _nbpErrorMessage: string = undefined;
    @Input()
    set nbpErrorMessage(value: string) {
        this._nbpErrorMessage = value;
        if (this.component) {
            this.component.nbpErrorMessage = value;
        }
    }
    get nbpErrorMessage(): string {
        return this._nbpErrorMessage;
    }


    private _showErrorLabel: boolean = undefined;
    @Input()
    set showErrorLabel(value: boolean) {
        this._showErrorLabel = value;
        if (this.component) {
            this.component.showErrorLabel = value;
        }
    }
    get showErrorLabel(): boolean {
        return this._showErrorLabel;
    }


    private _maxDate: Date = undefined;
    @Input()
    set maxDate(value: Date) {
        this._maxDate = value;
        if (this.component) {
            this.component.maxDate = value;
        }
    }
    get maxDate(): Date {
        return this._maxDate;
    }


    private _minDate: Date = undefined;
    @Input()
    set minDate(value: Date) {
        this._minDate = value;
        if (this.component) {
            this.component.minDate = value;
        }
    }
    get minDate(): Date {
        return this._minDate;
    }


    private _startDateEnabled: NgbDateStruct = undefined;
    @Input()
    set startDateEnabled(value: NgbDateStruct) {
        this._startDateEnabled = value;
        if (this.component) {
            this.component.startDateEnabled = value;
        }
    }
    get startDateEnabled(): NgbDateStruct {
        return this._startDateEnabled;
    }


    private _endDateEnabled: NgbDateStruct = undefined;
    @Input()
    set endDateEnabled(value: NgbDateStruct) {
        this._endDateEnabled = value;
        if (this.component) {
            this.component.endDateEnabled = value;
        }
    }
    get endDateEnabled(): NgbDateStruct {
        return this._endDateEnabled;
    }


    private _disabled: boolean = undefined;
    @Input()
    set disabled(value: boolean) {
        this._disabled = value;
        if (this.component) {
            this.component.disabled = value;
        }
    }
    get disabled(): boolean {
        return this._disabled;
    }


    private _required: boolean = undefined;
    @Input()
    set required(value: boolean) {
        this._required = value;
        if (this.component) {
            this.component.required = value;
        }
    }
    get required(): boolean {
        return this._required;
    }


    private _nbpStyle: NbpStyle = undefined;
    @Input()
    set nbpStyle(value: NbpStyle) {
        this._nbpStyle = value;
        if (this.component) {
            this.component.nbpStyle = value;
        }
    }
    get nbpStyle(): NbpStyle {
        return this._nbpStyle;
    }


    private __type: string = undefined;
    @Input()
    set _type(value: string) {
        this.__type = value;
        if (this.component) {
            this.component._type = value;
        }
    }
    get _type(): string {
        return this.__type;
    }


    private _disabledWeekend: boolean = undefined;
    @Input()
    set disabledWeekend(value: boolean) {
        this._disabledWeekend = value;
        if (this.component) {
            this.component.disabledWeekend = value;
        }
    }
    get disabledWeekend(): boolean {
        return this._disabledWeekend;
    }


    private _holidays: Array<NgbDateStruct> = undefined;
    @Input()
    set holidays(value: Array<NgbDateStruct>) {
        this._holidays = value;
        if (this.component) {
            this.component.holidays = value;
        }
    }
    get holidays(): Array<NgbDateStruct> {
        return this._holidays;
    }


    private _taxDeadlines: Array<NgbDateStruct> = undefined;
    @Input()
    set taxDeadlines(value: Array<NgbDateStruct>) {
        this._taxDeadlines = value;
        if (this.component) {
            this.component.taxDeadlines = value;
        }
    }
    get taxDeadlines(): Array<NgbDateStruct> {
        return this._taxDeadlines;
    }


    private _firstFocus: boolean = undefined;
    @Input()
    set firstFocus(value: boolean) {
        this._firstFocus = value;
        if (this.component) {
            this.component.firstFocus = value;
        }
    }
    get firstFocus(): boolean {
        return this._firstFocus;
    }


    private _focusOn: boolean = undefined;
    @Input()
    set focusOn(value: boolean) {
        this._focusOn = value;
        if (this.component) {
            this.component.focusOn = value;
        }
    }
    get focusOn(): boolean {
        return this._focusOn;
    }


    private _tabindex: undefined = undefined;
    @Input()
    set tabindex(value: undefined) {
        this._tabindex = value;
        if (this.component) {
            this.component.tabindex = value;
        }
    }
    get tabindex(): undefined {
        return this._tabindex;
    }


    private _label: string = undefined;
    @Input()
    set label(value: string) {
        this._label = value;
        if (this.component) {
            this.component.label = value;
        }
    }
    get label(): string {
        return this._label;
    }


    private _ariaLabel: string = undefined;
    @Input()
    set ariaLabel(value: string) {
        this._ariaLabel = value;
        if (this.component) {
            this.component.ariaLabel = value;
        }
    }
    get ariaLabel(): string {
        return this._ariaLabel;
    }


    private _nbpLightMode: boolean = undefined;
    @Input()
    set nbpLightMode(value: boolean) {
        this._nbpLightMode = value;
        if (this.component) {
            this.component.nbpLightMode = value;
        }
    }
    get nbpLightMode(): boolean {
        return this._nbpLightMode;
    }


    private _nbpLabelPattern: NbpCalendarPattern = undefined;
    @Input()
    set nbpLabelPattern(value: NbpCalendarPattern) {
        this._nbpLabelPattern = value;
        if (this.component) {
            this.component.nbpLabelPattern = value;
        }
    }
    get nbpLabelPattern(): NbpCalendarPattern {
        return this._nbpLabelPattern;
    }

    private _nbpDateSeparator: NbpDateSeparator = undefined;
    @Input()
    set nbpDateSeparator(value: NbpDateSeparator) {
        this._nbpDateSeparator = value;
        if (this.component) {
            this.component.nbpDateSeparator = value;
        }
    }
    get nbpDateSeparator(): NbpDateSeparator {
        return this._nbpDateSeparator;
    }


    private _opened: boolean = undefined;
    @Input()
    set opened(value: boolean) {
        this._opened = value;
        if (this.component) {
            this.component.opened = value;
        }
    }
    get opened(): boolean {
        return this._opened;
    }


    private _nbpPlacement: string = undefined;
    @Input()
    set nbpPlacement(value: string) {
        this._nbpPlacement = value;
        if (this.component) {
            this.component.nbpPlacement = value;
        }
    }
    get nbpPlacement(): string {
        return this._nbpPlacement;
    }

    private _enableDefaultViewOnUnifiedVG: boolean = undefined;
    @Input()
    set enableDefaultViewOnUnifiedVG(value: boolean) {
        this._enableDefaultViewOnUnifiedVG = value;
        if (this.component) {
            this.component.enableDefaultViewOnUnifiedVG = value;
        }
    }
    get enableDefaultViewOnUnifiedVG(): boolean {
        return this._enableDefaultViewOnUnifiedVG;
    }

    private _enableDynamicPosition: boolean = undefined;
    @Input()
    set enableDynamicPosition(value: boolean) {
        this._enableDynamicPosition = value;
        if (this.component) {
            this.component.enableDynamicPosition = value;
        }
    }
    get enableDynamicPosition(): boolean {
        return this._enableDynamicPosition;
    }


    @Output() nbpOnNavigate = new EventEmitter<{ current: any, next: any }>();

    @Output() modelChange = new EventEmitter<any>();

    @Output() focus = new EventEmitter<boolean>();

    get rplCalendar() {
        return this.component.rplCalendar;
    }

    _ngbModelChanged(event: NgbDateStruct) {
        return this.component ? this.component._ngbModelChanged(event): undefined;
    }

    getCalendarWidth() {
        return this.component.getCalendarWidth();
    }

    getCalendarHeight() {
        return this.component.getCalendarHeight();
    }

    public handleCalendar($event) {
        return this.component.handleCalendar($event);
    }
    public setAriaLabel(date: any) {
        return this.component.setAriaLabel(date);
    }
    public setSpanLabel(date){
        return this.component.setSpanLabel(date);
    }
    public focusOutComponent(event: any){
        return this.component.focusOutComponent(event);
    }

}
